@@ -11,10 +11,10 @@ from xml.dom import minidom
 import numpy as np
 import scipy.interpolate as scpinterp
 import scipy.integrate as scpintg
+import matplotlib.colors as mplcol
 from inspect import signature as insp
 
 
-
 # ToFu-specific
 try:
     import tofu.geom._def as _def
@@ -24,7 +24,7 @@ except Exception:
     from . import _GG as _GG
 
 
-_LTYPES = [int, float, np.int_, np.float_]
+_LTYPES = [int, float, np.int_, np.float64]
 _RES = 0.1
 
 
@@ -71,6 +71,7 @@ def _check_float(var=None, varname=None, vardef=None):
 def _get_pts_from_path_svg(
     path_str=None,
     res=None,
+    k0=None,
 ):
 
     # Check inputs
@@ -123,14 +124,20 @@ def _get_pts_from_path_svg(
     if 'z' not in path_str.lower():
         if pts.shape[1] == 2:
             isref = True
+        elif np.allclose(pts[:, 0], pts[:, -1]):
+            pass
         else:
+            pts = np.concatenate((pts, pts[:, 0:1]), axis=1)
             msg = (
-                "Non-conform path ({}) identified!\n"
-                + "All path must be either:\n"
-                + "\t- closed\n"
-                + "\t- or a unique straight line with 2 points\n"
+                f"Non-conform path '{k0}' identified!\n"
+                "All path must be either:\n"
+                "\t- closed\n"
+                "\t- or a unique straight line with 2 points\n"
+                "Provided:\n"
+                f"path_str: {path_str}\n"
+                f"  => closed automatically"
             )
-            raise Exception(msg)
+            warnings.warn(msg)
 
     return pts, isref
 
@@ -192,12 +199,13 @@ def get_paths_from_svg(
 
     # Derive usable data
     kstr = 'fill:'
+    dpath = {k0: v0 for k0, v0 in dpath.items() if kstr in v0['color']}
     lk = list(dpath.keys())
     ref = None
     for ii, k0 in enumerate(lk):
 
         v0 = dpath[k0]
-        poly, isref = _get_pts_from_path_svg(v0['poly'], res=res)
+        poly, isref = _get_pts_from_path_svg(v0['poly'], res=res, k0=k0)
         if isref is True:
             ref = poly
             del dpath[k0]
@@ -210,7 +218,10 @@ def get_paths_from_svg(
             dpath[k0]['cls'] = 'Ves'
             color = None
         else:
-            dpath[k0]['cls'] = 'PFC'
+            if mplcol.to_rgb(color) == (1., 0., 0.):
+                dpath[k0]['cls'] = 'CoilPF'
+            else:
+                dpath[k0]['cls'] = 'PFC'
         dpath[k0]['color'] = color
 
     # Check for negative r
@@ -286,12 +297,15 @@ def get_paths_from_svg(
     if verb is True:
         lVes = sorted([k0 for k0, v0 in dpath.items() if v0['cls'] == 'Ves'])
         lPFC = sorted([k0 for k0, v0 in dpath.items() if v0['cls'] == 'PFC'])
+        lCoilPF = sorted([
+            k0 for k0, v0 in dpath.items() if v0['cls'] == 'CoilPF'
+        ])
         lobj = [
             '\t- {}: {} ({} pts, {})'.format(
                 dpath[k0]['cls'], k0,
                 dpath[k0]['poly'].shape[1], dpath[k0]['color'],
             )
-            for k0 in lVes + lPFC
+            for k0 in lVes + lPFC + lCoilPF
         ]
         msg = (
             "The following structures were loaded:\n".format(pfe)
@@ -306,6 +320,8 @@ def get_paths_from_svg(
 # ==============================================================================
 # = Ves sub-functions
 # ==============================================================================
+
+
 def _Struct_set_Poly(
     Poly, pos=None, extent=None, arrayorder="C", Type="Tor", Clock=False
 ):
@@ -499,7 +515,7 @@ def _Ves_get_sample_checkinputs(
     dres = {'edge': 1, 'cross': 2, 'surface': 2, 'volume': 3}
     if res is None:
         res = _SAMPLE_RES[which]
-    ltypes = [int, float, np.int_, np.float_]
+    ltypes = [int, float, np.int_, np.float64]
     c0 = (type(res) in ltypes
           or (hasattr(res, "__iter__")
               and len(res) == dres[which]
@@ -561,13 +577,13 @@ def _Ves_get_sample_checkinputs(
     c0 = (ind is None
           or (isinstance(ind, np.ndarray)
               and ind.ndim == 1
-              and ind.dtype == np.int_
+              and 'int' in ind.dtype.name
               and np.all(ind >= 0))
           or (which == 'surface'
               and isinstance(ind, list)
               and all([isinstance(indi, np.ndarray)
                        and indi.ndim == 1
-                       and indi.dtype == np.int_
+                       and 'int' in indi.dtype.name
                        and np.all(indi >= 0) for indi in ind])))
     if not c0:
         msg = ("Arg ind must be either:\n"
@@ -578,6 +594,11 @@ def _Ves_get_sample_checkinputs(
         msg += "  You provided:\n{}".format(ind)
         raise Exception(msg)
 
+    if isinstance(ind, np.ndarray):
+        ind = ind.astype(np.int64)
+    elif isinstance(ind, list):
+        ind = [ii.astype(np.int64) for ii in ind]
+
     return res, domain, resMode, ind
 
 
@@ -604,7 +625,7 @@ def _Ves_get_sampleEdge(
     )
 
     # specific
-    ltypes = [int, float, np.int_, np.float_]
+    ltypes = [int, float, np.int_, np.float64]
     assert type(offsetIn) in ltypes
 
     # -------------
@@ -1069,7 +1090,7 @@ def _Struct_get_phithetaproj(ax=None, poly_closed=None, lim=None, noccur=0):
         nphi = np.r_[1]
     else:
         assert lim.ndim == 2, str(lim)
-        nphi = np.ones((noccur,), dtype=int)
+        nphi = np.ones((noccur,), dtype=np.int64)
         ind = (lim[:, 0] > lim[:, 1]).nonzero()[0]
         Dphi = np.concatenate((lim, np.full((noccur, 2), np.nan)), axis=1)
         if ind.size > 0:
@@ -1451,7 +1472,7 @@ def LOS_calc_signal(
     if method == "sum":
         Int = np.sum(Vals) * dLr
     elif method == "simps":
-        Int = scpintg.simps(Vals, x=None, dx=dLr)
+        Int = scpintg.simpson(Vals, x=None, dx=dLr)
     elif method == "romb":
         Int = scpintg.romb(Vals, dx=dLr, show=False)
-    return Int
+    return Int
\ No newline at end of file
