@@ -33,6 +33,15 @@ except Exception as err:
     raise Exception('imas not available')
 
 
+# Useful scalar types
+_NINT = (np.int32, np.int64)
+_INT = (int,) + _NINT
+_NFLOAT = (np.float32, np.float64)
+_FLOAT = (float,) + _NFLOAT
+_NUMB = _INT + _FLOAT
+_BOOL = (bool, np.bool_)
+
+
 _DSHORT = _defimas2tofu._dshort
 _DCOMP = _defimas2tofu._dcomp
 _DDUNITS = imas.dd_units.DataDictionaryUnits()
@@ -201,6 +210,7 @@ def get_fsig(sig):
 
                     if ind is None:
                         ind = range(0, nb)
+
                     if nsig > 1:
                         if isinstance(ind, str) or len(ind) != 1:
                             msg = ('ind should be have len() = 1\n'
@@ -208,7 +218,15 @@ def get_fsig(sig):
                             raise Exception(msg)
 
                     if len(ind) == 1:
-                        sig[jj] = sig[jj][ind[0]]
+                        if len(sig[jj]) < ind[0] + 1:
+                            msg = (
+                                f"dcond[{ii}]['ind'] = {dcond[ii]['ind']} "
+                                f"so ind = {ind} "
+                                f"but len(sig[{jj}]) = {len(sig[jj])}"
+                            )
+                            raise Exception(msg)
+                        else:
+                            sig[jj] = sig[jj][ind[0]]
                     else:
                         if nsig != 1:
                             msg = ("nsig should be 1!\n"
@@ -248,12 +266,20 @@ def get_fsig(sig):
                     sig[jj] = sig[jj][ind[0]]
 
         # Conditions for stacking / sqeezing sig
-        lc = [(stack and nsig > 1 and isinstance(sig[0], np.ndarray)
-               and all([ss.shape == sig[0].shape for ss in sig[1:]])),
-              (stack and nsig > 1
-               and type(sig[0]) in [int, float, np.int_, np.float_, str]),
-              (stack and nsig == 1
-               and type(sig) in [np.ndarray, list, tuple])]
+        lc = [
+            (
+                stack and nsig > 1 and isinstance(sig[0], np.ndarray)
+                and all([ss.shape == sig[0].shape for ss in sig[1:]])
+            ),
+            (
+                stack and nsig > 1
+                and isinstance(sig[0], _NUMB + (str,))
+            ),
+            (
+                stack and nsig == 1
+                and type(sig) in [np.ndarray, list, tuple]
+            ),
+        ]
 
         if lc[0]:
             sig = np.atleast_1d(np.squeeze(np.stack(sig)))
@@ -447,30 +473,98 @@ def _checkformat_getdata_occ(occ, ids, dids=None):
 
 
 def _checkformat_getdata_indch(indch, nch):
-    msg = ("Arg indch must be a either:\n"
-           + "    - None: all channels used\n"
-           + "    - int: channel to use (index)\n"
-           + "    - array of int: channels to use (indices)\n"
-           + "    - array of bool: channels to use (indices)\n")
-    lc = [indch is None,
-          isinstance(indch, int),
-          hasattr(indch, '__iter__') and not isinstance(indch, str)]
-    if not any(lc):
+    """ Check index of channels, returns array of int indices
+
+    Parameters
+    ----------
+    indch : None / int / aiterable of int or bool
+        Input index of channels
+    nch : int
+        Max number of channels
+
+    Raises
+    ------
+    Exception
+        DESCRIPTION.
+
+    Returns
+    -------
+    indch : np.ndarray of int
+        Output indices
+
+    """
+
+    # -----------------------------
+    # Initialize error msg
+    # -----------------------------
+
+    msg = (
+        "Arg indch must be a either:\n"
+        "\t- None: all channels used\n"
+        "\t- int: channel to use (index)\n"
+        "\t- array of int: channels to use (indices)\n"
+        "\t- array of bool: channels to use (indices)\n"
+    )
+
+    # -----------------------------
+    # List of acceptable conditions
+    # -----------------------------
+
+    lc0 = [
+        indch is None,
+        isinstance(indch, _INT),
+        hasattr(indch, '__iter__') and not isinstance(indch, str),
+    ]
+
+    if not any(lc0):
         raise Exception(msg)
-    if lc[0]:
+
+    # ------------------------
+    # None
+    # ------------------------
+
+    if lc0[0]:
+        # defaul to all indices
         indch = np.arange(0, nch)
-    elif lc[1] or lc[2]:
+
+    # ------------------------
+    # integer
+    # ------------------------
+
+    elif lc0[1]:
+        # make numpy int array
         indch = np.r_[indch].ravel()
-        lc = [indch.dtype == np.int, indch.dtype == np.bool]
-        if not any(lc):
+
+    # ------------------------
+    # iterable: int or bool
+    # ------------------------
+
+    elif lc0[2]:
+        # make numpy array
+        indch = np.r_[indch].ravel()
+
+        # get dtype (may also be a list)
+        lc1 = [
+            isinstance(indch[0], _INT),
+            isinstance(indch[0], _BOOL),
+        ]
+
+        if not any(lc1):
             raise Exception(msg)
-        if lc[1]:
+
+        if lc1[1]:
+            # convert from bool to int
             indch = np.nonzero(indch)[0]
+
+        # safety check
         if not np.all((indch >= 0) & (indch < nch)):
-            msg = ("Some channel indices are out of scope!\n"
-                   + "\t- nch: {}\n".format(nch)
-                   + "\t- indch: {}".format(indch))
+            msg = (
+                "Some channel indices are out of scope!\n"
+                f"\t- nch: {nch}\n"
+                f"\t- indch: {indch}"
+            )
             raise Exception(msg)
+
     return indch
 
 
@@ -513,14 +607,19 @@ def _check_data(data, pos=None, nan=None, isclose=None, empty=None):
     # All values larger than 1e30 are default imas values => nan
     if nan is True:
         for ii in range(0, len(data)):
-            c0 = (isinstance(data[ii], np.ndarray)
-                  and data[ii].dtype in [np.float_, np.int_])
+            c0 = (
+                isinstance(data[ii], np.ndarray)
+                and data[ii].dtype in _NUMB
+            )
             if c0 is True:
                 # Make sure to test only non-nan to avoid warning
                 ind = (~np.isnan(data[ii])).nonzero()
                 ind2 = np.abs(data[ii][ind]) > 1.e30
-                ind = tuple([ii[ind2] for ii in ind])
-                data[ii][ind] = np.nan
+                if np.any(ind2):
+                    ind = tuple([ii[ind2] for ii in ind])
+                    if data[ii].dtype in _INT:
+                        data[ii] = data[ii].astype(float)
+                    data[ii][ind] = np.nan
 
     # data supposed to be positive only (nan otherwise)
     if pos is True:
@@ -546,7 +645,7 @@ def _get_data_units(ids=None, sig=None, occ=None,
                     stack=None, isclose=None, flatocc=None,
                     nan=None, pos=None, empty=None,
                     dids=None, dcomp=None, dshort=None, dall_except=None,
-                    data=True, units=True):
+                    data=True, units=True, strict=None):
     """ Reference method for getting data and units, using shortcuts
 
     For a given ids, sig (shortcut) and occurence (occ)
@@ -599,7 +698,9 @@ def _get_data_units(ids=None, sig=None, occ=None,
                     data=True, units=False, indt=indt, stack=stack,
                     flatocc=False, nan=nan, pos=pos, warn=False,
                     dids=dids, dcomp=dcomp, dshort=dshort,
-                    dall_except=dall_except)[ids]
+                    dall_except=dall_except,
+                    strict=strict,
+                )[ids]
                 out = [dcomp[ids][sig]['func'](
                     *[ddata[kk]['data'][nn] for kk in lstr],
                     **kargs)
@@ -607,13 +708,13 @@ def _get_data_units(ids=None, sig=None, occ=None,
                 if pos is None:
                     pos = dcomp[ids][sig].get('pos', False)
             except Exception as err:
-                errdata = str(err)
+                errdata = err
 
         if units is True:
             try:
                 unit = dcomp[ids][sig].get('units', None)
             except Exception as err:
-                errunits = str(err)
+                errunits = err
 
     # Data available from ids
     else:
@@ -626,13 +727,13 @@ def _get_data_units(ids=None, sig=None, occ=None,
                 if pos is None:
                     pos = dshort[ids][sig].get('pos', False)
             except Exception as err:
-                errdata = str(err)
+                errdata = err
 
         if units is True:
             try:
                 unit = get_units(ids, sig)
             except Exception as err:
-                errunits = str(err)
+                errunits = err
 
     # Check data
     isempty = None
@@ -642,7 +743,7 @@ def _get_data_units(ids=None, sig=None, occ=None,
                                    isclose=isclose, empty=empty)
         if np.all(isempty):
             msg = ("empty data in {}.{}".format(ids, sig))
-            errdata = msg
+            errdata = Exception(msg)
         elif nocc == 1 and flatocc is True:
             out = out[0]
             isempty = isempty[0]
@@ -720,7 +821,9 @@ def get_data_units(dsig=None, occ=None,
                     data=data, units=units,
                     nan=nan, pos=pos, empty=empty,
                     dids=dids, dcomp=dcomp, dshort=dshort,
-                    dall_except=dall_except)
+                    dall_except=dall_except,
+                    strict=strict,
+                )
 
                 lc = [dout[ids][sigi]['errdata'] is not None,
                       dout[ids][sigi]['errunits'] is not None]
@@ -732,31 +835,41 @@ def get_data_units(dsig=None, occ=None,
                         dfail[ids][sigi+' units'] = dout[ids][sigi]['errunits']
 
             except Exception as err:
-                dfail[ids][sigi] = str(err)
+                dfail[ids][sigi] = err
                 anyfail = True
 
         if len(dfail[ids]) == 0:
             del dfail[ids]
 
+    # ---------------------
     # Print if any failure
+
     if anyfail:
+
+        if strict is True:
+            for ids, vids in dfail.items():
+                for sigi, verr in vids.items():
+                    print(f"\n{ids}\t{sigi}\t{verr}")
+                    raise verr
+
         if data is True:
             for ids in dfail.keys():
                 for sigi in list(dout[ids].keys()):
                     if dout[ids][sigi]['errdata'] is not None:
                         del dout[ids][sigi]
+
         if warn:
             msg = "The following data could not be retrieved:"
             for ids in dfail.keys():
                 nmax = np.max([len(k1) for k1 in dfail[ids].keys()])
                 msg += "\n\t- {}:".format(ids)
-                for sigi, msgi in dfail[ids].items():
-                    msg += "\n\t\t{0}:  {1}".format(sigi.ljust(nmax),
-                                                    msgi.replace('\n', ' '))
+                for sigi, erri in dfail[ids].items():
+                    msgi = str(erri).replace('\n', ' ')
+                    msg += f"\n\t\t{sigi.ljust(nmax)}:  {msgi}"
             warnings.warn(msg)
 
     # return
     if return_all:
         return dout, dfail, dsig
     else:
-        return dout
+        return dout
\ No newline at end of file
